/******************************************************************************
 THXAboutDialog.cc

	BASE CLASS = JXDialogDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXAboutDialog.h"
#include "thxHelpText.h"
#include "thxPixmaps.h"
#include <JXApplication.h>
#include <JXWindow.h>
#include <JXTextButton.h>
#include <JXStaticText.h>
#include <JXImageWidget.h>
#include <JXImage.h>
#include <JXHelpManager.h>
#include <jXGlobals.h>
#include <jAssert.h>

extern const JCharacter* kTHXVersionStr;

static const JCharacter* kUpgradeNotice     = "\n\n\nYou have just upgraded from version ";
static const JCharacter* kChangeButtonLabel = "Changes";

/******************************************************************************
 Constructor

	If prevVersStr is not empty, we change the Help button to "Changes".

 ******************************************************************************/

THXAboutDialog::THXAboutDialog
	(
	JXDirector*			supervisor,
	const JCharacter*	prevVersStr
	)
	:
	JXDialogDirector(supervisor, kTrue)
{
	itsIsUpgradeFlag = kFalse;

	BuildWindow(prevVersStr);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXAboutDialog::~THXAboutDialog()
{
}

/******************************************************************************
 BuildWindow (private)

 ******************************************************************************/

void
THXAboutDialog::BuildWindow
	(
	const JCharacter* prevVersStr
	)
{
// begin JXLayout

    JXWindow* window = new JXWindow(this, 370,115, "");
    assert( window != NULL );
    SetWindow(window);

    JXImageWidget* imageWidget =
        new JXImageWidget(window,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,20, 40,40);
    assert( imageWidget != NULL );

    JXStaticText* textWidget =
        new JXStaticText("", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 70,20, 280,50);
    assert( textWidget != NULL );

    JXTextButton* okButton =
        new JXTextButton("OK", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 259,84, 62,22);
    assert( okButton != NULL );
    okButton->SetShortcuts("^M");

    itsCreditsButton =
        new JXTextButton("Credits", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 50,85, 60,20);
    assert( itsCreditsButton != NULL );

    itsHelpButton =
        new JXTextButton("Help", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 155,85, 60,20);
    assert( itsHelpButton != NULL );

// end JXLayout

	window->SetTitle("About");
	SetButtons(okButton, NULL);

	ListenTo(itsHelpButton);
	ListenTo(itsCreditsButton);

	JString text = kTHXVersionStr;
	if (!JStringEmpty(prevVersStr))
		{
		text += kUpgradeNotice;
		text += prevVersStr;
		itsHelpButton->SetLabel(kChangeButtonLabel);
		itsIsUpgradeFlag = kTrue;
		}
	textWidget->SetText(text);

	JXImage* image = new JXImage(GetDisplay(), window->GetColormap(), THXGetAboutIcon());
	assert( image != NULL );
	image->SetDefaultState(JXImage::kRemoteStorage);
	image->ConvertToDefaultState();
	imageWidget->SetImage(image, kTrue);

	const JSize bdh = textWidget->GetBoundsHeight();
	const JSize aph = textWidget->GetApertureHeight();
	if (bdh > aph)
		{
		window->AdjustSize(0, bdh - aph);	// safe to calculate once bdh > aph
		}
}

/******************************************************************************
 Receive (protected)

	Handle extra buttons.

 ******************************************************************************/

void
THXAboutDialog::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsHelpButton && message.Is(JXButton::kPushed))
		{
		if (itsIsUpgradeFlag)
			{
			(JXGetHelpManager())->ShowSection(kTHXChangeLogName);
			}
		else
			{
			(JXGetHelpManager())->ShowSection(kTHXOverviewHelpName);
			}
		EndDialog(kTrue);
		}

	else if (sender == itsCreditsButton && message.Is(JXButton::kPushed))
		{
		(JXGetHelpManager())->ShowSection(kTHXCreditsName);
		EndDialog(kTrue);
		}

	else
		{
		JXDialogDirector::Receive(sender, message);
		}
}
