/******************************************************************************
 THXApp.h

	Interface for the THXApp class

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#ifndef _H_THXApp
#define _H_THXApp

#include <JXApplication.h>

class JString;
class JXTextMenu;
class THXVarList;
class THXVarDirector;
class THXExprDirector;
class THXPlotDirector;
class THXPlotFunctionDialog;
class THXBaseConvDirector;

class THXApp : public JXApplication
{
public:

	static const JCharacter* kAppSignature;

public:

	THXApp(int* argc, char* argv[]);

	virtual ~THXApp();

	void	DisplayAbout(const JCharacter* prevVersStr = NULL);

	const THXVarList*	GetVariableList() const;
	THXExprDirector*	NewExpression(const JBoolean centerOnScreen = kFalse);
	void				NewPlot(const THXPlotDirector* prevPlot = NULL);
	void				ShowConstants() const;
	void				ShowBaseConversion() const;

	void	BuildPlotMenu(JXTextMenu* menu, const THXPlotDirector* origPlot,
						  JIndex* initialChoice) const;

	JBoolean	KeyPadIsVisible() const;
	void		SetKeyPadVisible(const JBoolean visible);
	void		ToggleKeyPadVisible();

protected:

	virtual JBoolean	Close();
	virtual void		CleanUpBeforeSuddenDeath(const JXDocumentManager::SafetySaveReason reason);
	virtual void		DirectorClosed(JXDirector* theDirector);
	virtual void		Receive(JBroadcaster* sender, const Message& message);

private:

	JString*	itsStatePath;

	THXVarList*					itsVarList;
	THXVarDirector*				itsVarDirector;
	JPtrArray<THXExprDirector>*	itsExprList;
	JBoolean					itsKeyPadVisibleFlag;
	JPtrArray<THXPlotDirector>*	itsPlotList;
	THXPlotFunctionDialog*		itsPlotFnDialog;	// NULL unless asking user
	THXBaseConvDirector*		itsBCDirector;

private:

	void	RestoreProgramState();
	void	InitProgramState();
	void	SaveProgramState();

	void	CreatePlot();

	static JOrderedSetT::CompareResult
		ComparePlotTitles(THXPlotDirector* const & p1, THXPlotDirector* const & p2);

	// not allowed

	THXApp(const THXApp& source);
	const THXApp& operator=(const THXApp& source);
};


/******************************************************************************
 GetVariableList

 ******************************************************************************/

inline const THXVarList*
THXApp::GetVariableList()
	const
{
	return itsVarList;
}

/******************************************************************************
 KeyPad visible

 ******************************************************************************/

inline JBoolean
THXApp::KeyPadIsVisible()
	const
{
	return itsKeyPadVisibleFlag;
}

inline void
THXApp::ToggleKeyPadVisible()
{
	SetKeyPadVisible(JNegate(itsKeyPadVisibleFlag));
}

#endif
