/******************************************************************************
 THXKeyPad.cc

	BASE CLASS = JXWidgetSet

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXKeyPad.h"
#include "THXExprEditor.h"
#include <JXTextButton.h>
#include <jASCIIConstants.h>
#include <jAssert.h>

// remember to keep the enum up to date

static const JCharacter kOpKey[] =
{
	'.', 'e', '+', '-', '*', '/', '=', kJEscapeKey
};

/******************************************************************************
 Constructor

 ******************************************************************************/

THXKeyPad::THXKeyPad
	(
	JXContainer*		enclosure,
	const HSizingOption	hSizing,
	const VSizingOption	vSizing,
	const JCoordinate	x,
	const JCoordinate	y,
	const JCoordinate	w,
	const JCoordinate	h
	)
	:
	JXWidgetSet(enclosure, hSizing, vSizing, x,y, w,h)
{
	itsExpr = NULL;
	CreateButtons();
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXKeyPad::~THXKeyPad()
{
}

/******************************************************************************
 CreateButtons (private)

 ******************************************************************************/

void
THXKeyPad::CreateButtons()
{
// begin kpLayout

    itsDigitButton[7] =
        new JXTextButton("7", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,0, 20,20);
    assert( itsDigitButton[7] != NULL );

    itsDigitButton[8] =
        new JXTextButton("8", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,0, 20,20);
    assert( itsDigitButton[8] != NULL );

    itsDigitButton[9] =
        new JXTextButton("9", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 40,0, 20,20);
    assert( itsDigitButton[9] != NULL );

    itsDigitButton[4] =
        new JXTextButton("4", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,20, 20,20);
    assert( itsDigitButton[4] != NULL );

    itsDigitButton[5] =
        new JXTextButton("5", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,20, 20,20);
    assert( itsDigitButton[5] != NULL );

    itsDigitButton[6] =
        new JXTextButton("6", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 40,20, 20,20);
    assert( itsDigitButton[6] != NULL );

    itsDigitButton[1] =
        new JXTextButton("1", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,40, 20,20);
    assert( itsDigitButton[1] != NULL );

    itsDigitButton[2] =
        new JXTextButton("2", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,40, 20,20);
    assert( itsDigitButton[2] != NULL );

    itsDigitButton[3] =
        new JXTextButton("3", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 40,40, 20,20);
    assert( itsDigitButton[3] != NULL );

    itsDigitButton[0] =
        new JXTextButton("0", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,60, 20,20);
    assert( itsDigitButton[0] != NULL );

    itsOpButton[kPtOpIndex] =
        new JXTextButton(".", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,60, 20,20);
    assert( itsOpButton[kPtOpIndex] != NULL );

    itsOpButton[kExpOpIndex] =
        new JXTextButton("E", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 40,60, 20,20);
    assert( itsOpButton[kExpOpIndex] != NULL );

    itsClearAllButton =
        new JXTextButton("Clear all", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,140, 60,20);
    assert( itsClearAllButton != NULL );

    itsOpButton[kClearOpIndex] =
        new JXTextButton("Clear", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,120, 60,20);
    assert( itsOpButton[kClearOpIndex] != NULL );

    itsOpButton[kEvalOpIndex] =
        new JXTextButton("=", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 40,80, 20,40);
    assert( itsOpButton[kEvalOpIndex] != NULL );

    itsOpButton[kMultiplyOpIndex] =
        new JXTextButton("\xD7", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,100, 20,20);
    assert( itsOpButton[kMultiplyOpIndex] != NULL );

    itsOpButton[kDivideOpIndex] =
        new JXTextButton("\xF7", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,100, 20,20);
    assert( itsOpButton[kDivideOpIndex] != NULL );

    itsOpButton[kAddOpIndex] =
        new JXTextButton("+", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 0,80, 20,20);
    assert( itsOpButton[kAddOpIndex] != NULL );

    itsOpButton[kSubtractOpIndex] =
        new JXTextButton("-", this,
                    JXWidget::kFixedLeft, JXWidget::kFixedTop, 20,80, 20,20);
    assert( itsOpButton[kSubtractOpIndex] != NULL );

// end kpLayout

JIndex i;

	for (i=0; i<10; i++)
		{
		ListenTo(itsDigitButton[i]);
		}

	for (i=0; i<kOpCount; i++)
		{
		ListenTo(itsOpButton[i]);
		}

	ListenTo(itsClearAllButton);
}

/******************************************************************************
 Receive (virtual protected)

 ******************************************************************************/

void
THXKeyPad::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (message.Is(JXButton::kPushed) && HandleButton(sender))
		{
		// work is done by function
		}
	else
		{
		JXWidgetSet::Receive(sender, message);
		}
}

/******************************************************************************
 HandleButton (private)

 ******************************************************************************/

JBoolean
THXKeyPad::HandleButton
	(
	JBroadcaster* sender
	)
	const
{
JIndex i;
JXKeyModifiers modifiers;

	if (itsExpr == NULL || !itsExpr->Focus())
		{
		return kFalse;
		}

	for (i=0; i<10; i++)
		{
		if (sender == itsDigitButton[i])
			{
			if (!itsExpr->HasSelection() && !itsExpr->UIFIsActive())
				{
				if (itsExpr->ContainsUIF())
					{
					itsExpr->ActivateNextUIF();
					}
				else
					{
					itsExpr->SelectAll();
					}
				}

			itsExpr->HandleKeyPress('0' + i, modifiers);
			return kTrue;
			}
		}

	for (i=0; i<kOpCount; i++)
		{
		if (sender == itsOpButton[i])
			{
			if (!itsExpr->HasSelection() && !itsExpr->UIFIsActive())
				{
				itsExpr->SelectAll();
				}

			itsExpr->HandleKeyPress(kOpKey[i], modifiers);
			return kTrue;
			}
		}

	if (sender == itsClearAllButton)
		{
		itsExpr->CancelEditing();
		itsExpr->SelectAll();
		itsExpr->DeleteSelection();
		return kTrue;
		}

	return kFalse;
}
