/******************************************************************************
 THXPlotFunctionDialog.cc

	BASE CLASS = JXDialogDirector

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXPlotFunctionDialog.h"
#include "THXVarList.h"
#include "thxGlobals.h"
#include <JXWindow.h>
#include <JXStaticText.h>
#include <JXTextButton.h>
#include <JXTextMenu.h>
#include <JXFloatInput.h>
#include <JXExprEditorSet.h>
#include <JXExprEditor.h>
#include <jAssert.h>

/******************************************************************************
 Constructor

	prevPlot can be NULL.

 ******************************************************************************/

THXPlotFunctionDialog::THXPlotFunctionDialog
	(
	JXDirector*				supervisor,
	const THXVarList*		varList,
	const THXPlotDirector*	prevPlot
	)
	:
	JXDialogDirector(supervisor, kTrue)
{
	BuildWindow(varList, prevPlot);
}

/******************************************************************************
 Destructor

 ******************************************************************************/

THXPlotFunctionDialog::~THXPlotFunctionDialog()
{
}

/******************************************************************************
 BuildWindow (protected)

 ******************************************************************************/

void
THXPlotFunctionDialog::BuildWindow
	(
	const THXVarList*		varList,
	const THXPlotDirector*	prevPlot
	)
{
// begin JXLayout

    JXWindow* window = new JXWindow(this, 330,320, "");
    assert( window != NULL );
    SetWindow(window);

    JXExprEditorSet* obj1 =
        new JXExprEditorSet(varList, JXMenu::kMacintoshStyle, &itsExprWidget, window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 0,0, 330,150);
    assert( obj1 != NULL );

    JXTextButton* cancelButton =
        new JXTextButton("Cancel", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 60,290, 60,20);
    assert( cancelButton != NULL );

    JXTextButton* okButton =
        new JXTextButton("OK", window,
                    JXWidget::kHElastic, JXWidget::kVElastic, 199,289, 72,22);
    assert( okButton != NULL );
    okButton->SetShortcuts("^M");

    itsPlotMenu =
        new JXTextMenu("Window to use:", window,
                    JXWidget::kHElastic, JXWidget::kFixedBottom, 30,170, 260,30);
    assert( itsPlotMenu != NULL );

    JXStaticText* obj2 =
        new JXStaticText("Curve name:", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 30,220, 80,20);
    assert( obj2 != NULL );

    itsCurveName =
        new JXInputField(window,
                    JXWidget::kHElastic, JXWidget::kFixedBottom, 110,220, 180,20);
    assert( itsCurveName != NULL );

    JXStaticText* obj3 =
        new JXStaticText("Initial range:", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 30,250, 80,20);
    assert( obj3 != NULL );

    itsMinValue =
        new JXFloatInput(window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 110,250, 80,20);
    assert( itsMinValue != NULL );

    JXStaticText* obj4 =
        new JXStaticText("to", window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 190,250, 20,20);
    assert( obj4 != NULL );

    itsMaxValue =
        new JXFloatInput(window,
                    JXWidget::kFixedLeft, JXWidget::kFixedBottom, 210,250, 80,20);
    assert( itsMaxValue != NULL );

// end JXLayout

	window->SetTitle("Plot function");
	SetButtons(okButton, cancelButton);

	(THXGetApplication())->BuildPlotMenu(itsPlotMenu, prevPlot, &itsPlotIndex);
	itsPlotMenu->SetToPopupChoice(kTrue, itsPlotIndex);
	ListenTo(itsPlotMenu);

	itsCurveName->SetText("New curve");
	itsCurveName->SetIsRequired();
	itsCurveName->ShareEditMenu(itsExprWidget->GetEditMenu(),
								JXMenu::kMacintoshStyle, kFalse, kFalse);

	itsMinValue->SetValue(0.0);
	itsMinValue->ShareEditMenu(itsExprWidget->GetEditMenu(),
							   JXMenu::kMacintoshStyle, kFalse, kFalse);

	itsMaxValue->SetValue(1.0);
	itsMaxValue->ShareEditMenu(itsExprWidget->GetEditMenu(),
							   JXMenu::kMacintoshStyle, kFalse, kFalse);

	itsExprWidget->SetBackColor(itsExprWidget->GetFocusColor());
}

/******************************************************************************
 GetSettings

 ******************************************************************************/

void
THXPlotFunctionDialog::GetSettings
	(
	JIndex*				plotIndex,
	const JFunction**	f,
	JString*			curveName,
	JFloat*				fMin,
	JFloat*				fMax
	)
	const
{
	*plotIndex = itsPlotIndex;
	*f         = itsExprWidget->GetFunction();
	*curveName = itsCurveName->GetText();

	JBoolean ok = itsMinValue->GetValue(fMin);
	assert( ok );

	ok = itsMaxValue->GetValue(fMax);
	assert( ok );
}

/******************************************************************************
 Receive (protected)

 ******************************************************************************/

void
THXPlotFunctionDialog::Receive
	(
	JBroadcaster*	sender,
	const Message&	message
	)
{
	if (sender == itsPlotMenu && message.Is(JXMenu::kNeedsUpdate))
		{
		itsPlotMenu->CheckItem(itsPlotIndex);
		}
	else if (sender == itsPlotMenu && message.Is(JXMenu::kItemSelected))
		{
		const JXMenu::ItemSelected* selection =
			dynamic_cast(const JXMenu::ItemSelected*, &message);
		assert( selection != NULL );
		itsPlotIndex = selection->GetIndex();
		}

	else
		{
		JXDialogDirector::Receive(sender, message);
		}
}

/******************************************************************************
 OKToDeactivate (virtual protected)

 ******************************************************************************/

JBoolean
THXPlotFunctionDialog::OKToDeactivate()
{
	if (!JXDialogDirector::OKToDeactivate())
		{
		return kFalse;
		}
	else if (Cancelled())
		{
		return kTrue;
		}

	else if (itsExprWidget->ContainsUIF())
		{
		(JGetUserNotification())->ReportError("Please finish entering the function.");
		itsExprWidget->Focus();
		return kFalse;
		}
	else
		{
		return kTrue;
		}
}
