/******************************************************************************
 thx-1138.cc

	Calculator and function plotter.

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "THXApp.h"
#include "THXMDIServer.h"
#include <string.h>
#include <stdlib.h>
#include <jAssert.h>

// Constants

const JCharacter* kTHXVersionNumberStr = "1.0.5";

const JCharacter* kTHXVersionStr =

	"THX-1138 1.0.5\n"
	"\n"
	"Electronic paper for calculations and graphs\n"
	"\n"
	"Copyright  1998-99 by John Lindal.\n"
	"This program may be freely distributed at no charge.";

// Prototypes

void ParseTextOptions(const int argc, char* argv[]);

void PrintHelp();
void PrintVersion();

/******************************************************************************
 main

 ******************************************************************************/

int
main
	(
	int		argc,
	char*	argv[]
	)
{
	ParseTextOptions(argc, argv);

	if (!THXMDIServer::WillBeMDIServer(THXApp::kAppSignature, argc, argv))
		{
		return 0;
		}

	THXApp* app = new THXApp(&argc, argv);
	assert( app != NULL );

	app->Run();
	return 0;
}

/******************************************************************************
 ParseTextOptions

	Handle the command line options that don't require opening an X display.

	Options that don't call exit() should use JXApplication::RemoveCmdLineOption()
	so ParseXOptions() won't gag.

 ******************************************************************************/

void
ParseTextOptions
	(
	const int	argc,
	char*		argv[]
	)
{
	long index = 1;
	while (index < argc)
		{
		if (strcmp(argv[index], "-h") == 0 ||
			strcmp(argv[index], "--help") == 0)
			{
			PrintHelp();
			exit(0);
			}
		else if (strcmp(argv[index], "-v") == 0 ||
				 strcmp(argv[index], "--version") == 0)
			{
			PrintVersion();
			exit(0);
			}
		index++;
		}
}

/******************************************************************************
 PrintHelp

 ******************************************************************************/

void
PrintHelp()
{
	cout << endl;
	cout << "THX-1138 " << kTHXVersionNumberStr << endl;
	cout << endl;
	cout << "This X application provides electronic paper for evaluating algebraic" << endl;
	cout << "expressions and plotting functions." << endl;
	cout << endl;
	cout << "-h  prints help" << endl;
	cout << "-v  prints version information" << endl;
	cout << endl;
}

/******************************************************************************
 PrintVersion

 ******************************************************************************/

void
PrintVersion()
{
	cout << endl;
	cout << kTHXVersionStr << endl;
	cout << endl;
}
