/******************************************************************************
 thxGlobals.cc

	Access to global objects and factories.

	Copyright  1998 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "thxGlobals.h"
#include "THXMDIServer.h"
#include "thxHelpText.h"
#include <JXDisplay.h>
#include <JXWindow.h>
#include <JXPTPrinter.h>
#include <JXPSPrinter.h>
#include <jAssert.h>

static THXApp*			theApplication   = NULL;	// owns itself
static THXMDIServer*	theMDIServer     = NULL;	// owned by JX
static JXPTPrinter*		theTapePrinter   = NULL;
static JXPSPrinter*		theGraphPrinter  = NULL;

static const JCharacter* kWMClassInstance = "THX-1138";
static const JCharacter* kExprWindowClass = "THX-1138_Expression";
static const JCharacter* kPlotWindowClass = "THX-1138_Plot";

/******************************************************************************
 THXCreateGlobals

 ******************************************************************************/

void
THXCreateGlobals
	(
	THXApp*				app,
	const JCharacter*	appSignature
	)
{
	theApplication = app;

	JXInitHelp(kTHXTOCHelpName, kTHXHelpSectionCount,
			   kTHXHelpSectionName, kTHXHelpSectionTitle, kTHXHelpSectionText,
			   JXMenu::kMacintoshStyle);

	theMDIServer = new THXMDIServer(appSignature);
	assert( theMDIServer != NULL );

	theTapePrinter = new JXPTPrinter;
	assert( theTapePrinter != NULL );

	JXDisplay* display = app->GetDisplay(1);
	theGraphPrinter = new JXPSPrinter(display, display->GetColormap());
	assert( theGraphPrinter != NULL );
}

/******************************************************************************
 THXDeleteGlobals

 ******************************************************************************/

void
THXDeleteGlobals()
{
	delete theTapePrinter;
	theTapePrinter = NULL;

	delete theGraphPrinter;
	theGraphPrinter = NULL;

	theApplication = NULL;
	theMDIServer   = NULL;
}

/******************************************************************************
 THXGetApplication

 ******************************************************************************/

THXApp*
THXGetApplication()
{
	assert( theApplication != NULL );
	return theApplication;
}

/******************************************************************************
 THXGetMDIServer

 ******************************************************************************/

THXMDIServer*
THXGetMDIServer()
{
	assert( theMDIServer != NULL );
	return theMDIServer;
}

/******************************************************************************
 THXGetGraphPrinter

 ******************************************************************************/

JXPSPrinter*
THXGetGraphPrinter()
{
	assert( theGraphPrinter != NULL );
	return theGraphPrinter;
}

/******************************************************************************
 THXGetTapePrinter

 ******************************************************************************/

JXPTPrinter*
THXGetTapePrinter()
{
	assert( theTapePrinter != NULL );
	return theTapePrinter;
}

/******************************************************************************
 THXGetWMClassInstance

 ******************************************************************************/

const JCharacter*
THXGetWMClassInstance()
{
	return kWMClassInstance;
}

/******************************************************************************
 THXGetExprWindowClass

 ******************************************************************************/

const JCharacter*
THXGetExprWindowClass()
{
	return kExprWindowClass;
}

/******************************************************************************
 THXGetPlotWindowClass

 ******************************************************************************/

const JCharacter*
THXGetPlotWindowClass()
{
	return kPlotWindowClass;
}
